/********************************************************************
 *
 * Beispiel-Programm zum Darstellen des Mauszeigers
 *  - Aufruf: dumpcursor <cursor> [<xpos> <ypos>]
 *
 * Quelle: BREDEX GmbH, Braunschweig
 *
 ********************************************************************/

/* Header-Dateien
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/CurUtil.h>
#define CURSORFONT "cursor"

/* Vorwaerts-Deklarationen
 */
void drawCursor();


/* globale Variablen
 */
Display* display;               /* Display */
int      screenNr;              /* Bildschirm-Nummer */
Window   root;                  /* Root-Window */
Font     cursorfont;            /* Cursor-Font */
GC       fgGC, bgGC;            /* Cursor-GCs */


main (argc, argv)
int argc;
char* argv[];
{
    int cursorindex;
    int x, y;

    if (argc != 4 && argc != 2) {
        fprintf (stderr, "Aufruf: %s <cursor> [<x-pos> <x-pos>]\n", argv[0]);
        exit (1);
    }

    /* Display oeffnen (Verbindung zum X-Server herstellen)
     */
    display = XOpenDisplay ((char*)NULL);
    if (display == NULL) {
        fprintf (stderr, "kann Display \"%s\" nicht oeffnen\n", 
			 XDisplayName((char*)NULL));
        exit (1);
    }
    screenNr = DefaultScreen (display);
    root     = RootWindow (display, screenNr);

    /* Cursor-Font laden
     */
    cursorfont = XLoadFont (display, CURSORFONT);
    if (cursorfont == None) {
        fprintf (stderr, "Cursor-Font \"%s\" nicht da\n", CURSORFONT);
        exit (1);
    }

    /* GCs erzeugen
     */
    {
      XGCValues gcv;
      unsigned long mask;

      mask = GCForeground|GCFont|GCSubwindowMode;
      gcv.font = cursorfont;
      gcv.subwindow_mode = IncludeInferiors;
      gcv.foreground = BlackPixel (display, screenNr);
      fgGC = XCreateGC (display, root, mask, &gcv);
      gcv.foreground = WhitePixel (display, screenNr);
      bgGC = XCreateGC (display, root, mask, &gcv);
    }

    /* Cursor-Index oder Cursor-Name uebergeben ?
     */
    if (isdigit(argv[1][0])) {
        cursorindex = atoi(argv[1]);
    }
    else {
        if (strncmp(argv[1],"XC_",3)==0 || strncmp(argv[1],"xc_",3)==0) {
            cursorindex = XmuCursorNameToIndex (argv[1]+3);
        }
        else {
            cursorindex = XmuCursorNameToIndex (argv[1]);
        }
        if (cursorindex < 0) {
            fprintf (stderr, "falscher Cursor-Name: \"%s\"\n", argv[1]);
            exit (1);
        }
    }

    /* Ausgabe-Position uebergeben ? 
     */
    if (argc == 4) {
        /* JA: nehmen
         */
        x = atoi(argv[2]);
        y = atoi(argv[3]);
    }
    else {
        /* NEIN: Mausposition nehmen
         */
        Window win;
        int i;

        printf ("Positioniere Maus (1 Sekunde)\n");
        sleep (1);
        XQueryPointer (display, root, &win, &win, &x, &y, &i, &i, &i);
    }

    /* Cursor an Position ausgeben */
    drawCursor (x, y, cursorindex);

    /* damit es auch gemacht wird
     */
    XSync (display, False);
}

void drawCursor (x, y, shape)
int x, y;
int shape;
{
    char c;

    /* Cursor-Vorder- und -Hintergrund jeweils als ein Zeichen ausgeben
     */
    c = (char)(shape+1);
    XDrawString (display, root, bgGC, x, y, &c, 1);
    c = (char)shape;
    XDrawString (display, root, fgGC, x, y, &c, 1);
}


